import contextlib
import unittest
import io
import re
import unittest.mock
from unittest import TextTestRunner

class MainTest(unittest.TestCase):

    def test_answer(self):
        capturedOutput = io.StringIO()                 
        sys.stdout = capturedOutput                     
        main()
        sys.stdout = sys.__stdout__ 

        checkQuery = ['PAN NYC CAS AMS HEL 283.7 kg']
        modelAnswerLength = len(checkQuery)

        output = list(capturedOutput.getvalue().strip().split("\n"))
        if output[-1] == "":
            output.pop()
        
        self.assertFalse(len(output) > modelAnswerLength, "Too many route combinations. You only need to print the route with the minimum emmisions!")

        # # Check output format
        # # Start: 5 ports with 3 routes each, separated by a space
        # # End: with x.x kg
        r = re.compile('^((\w{3} ){5})(\d+.\d kg)$') 
        isFormatCorrect = all(r.match(line) for line in output)
        self.assertTrue(isFormatCorrect, "The output format is incorrect. Please take a look at the example output.")
        
        # Default Test
        # Check if output and checkQuery contain the same elements irrespective of order
        self.assertTrue(sorted(output) == sorted(checkQuery), "Your answer is incorrect.")

test_suite = unittest.TestLoader().loadTestsFromTestCase(MainTest)
with io.StringIO() as buf:
    with contextlib.redirect_stdout(buf):
        TextTestRunner(stream=buf).run(test_suite)
    printTest(buf.getvalue())